#!/usr/bin/env python
"""
Unit tests for ExecuteDAO function(s)
"""

import unittest

from WMCore.Database.ExecuteDAO import ExecuteDAO, loggerSetup


class ExecuteDAOTest(unittest.TestCase):
    """
    unittest for ExecuteDAO functions
    """

    def setUp(self):
        self.maxDiff = None
        self.logger = loggerSetup()
        self.package = "WMCore.WMBS"
        self.module = "Workflow.GetDeletableWorkflows"
        self.daoObject = ExecuteDAO(logger=self.logger,
                                    package=self.package, daoModule=self.module)
        self.sqlQueriesResult = {}
        self.sqlQueriesExpected = {'compltetedWFs': """ SELECT name FROM wmbs_workflow
                            WHERE name NOT IN (
                                SELECT DISTINCT ww.name FROM wmbs_workflow ww
                                      INNER JOIN wmbs_subscription ws
                                         ON ws.workflow = ww.id
                                WHERE ws.finished =0)
                      """,
        'wfsWithIncompletedChildWFs': """ SELECT DISTINCT ww.name FROM wmbs_workflow ww
                                  INNER JOIN wmbs_subscription ws
                                      ON ws.workflow = ww.id
                                  INNER JOIN wmbs_fileset wfs ON
                                     wfs.id = ws.fileset
                                  INNER JOIN wmbs_fileset_files wfsf ON
                                     wfsf.fileset = wfs.id
                                  INNER JOIN wmbs_file_parent wfp ON
                                     wfp.parent = wfsf.fileid
                                  INNER JOIN wmbs_fileset_files child_fileset ON
                                     child_fileset.fileid = wfp.child
                                  INNER JOIN wmbs_subscription child_subscription ON
                                     child_subscription.fileset = child_fileset.fileset
                                  WHERE child_subscription.finished = 0
                            """,
        'sql': """SELECT DISTINCT wmbs_workflow.name, wmbs_workflow.spec,
                        wmbs_workflow.id AS workflow_id, wmbs_subscription.id AS sub_id
                 FROM wmbs_subscription
                     INNER JOIN wmbs_workflow ON
                         wmbs_workflow.id = wmbs_subscription.workflow
                     INNER JOIN ( SELECT name FROM wmbs_workflow
                            WHERE name NOT IN (
                                SELECT DISTINCT ww.name FROM wmbs_workflow ww
                                      INNER JOIN wmbs_subscription ws
                                         ON ws.workflow = ww.id
                                WHERE ws.finished =0)
                      ) complete_workflow ON
                         complete_workflow.name = wmbs_workflow.name
                 WHERE wmbs_workflow.name NOT IN ( SELECT DISTINCT ww.name FROM wmbs_workflow ww
                                  INNER JOIN wmbs_subscription ws
                                      ON ws.workflow = ww.id
                                  INNER JOIN wmbs_fileset wfs ON
                                     wfs.id = ws.fileset
                                  INNER JOIN wmbs_fileset_files wfsf ON
                                     wfsf.fileset = wfs.id
                                  INNER JOIN wmbs_file_parent wfp ON
                                     wfp.parent = wfsf.fileid
                                  INNER JOIN wmbs_fileset_files child_fileset ON
                                     child_fileset.fileid = wfp.child
                                  INNER JOIN wmbs_subscription child_subscription ON
                                     child_subscription.fileset = child_fileset.fileset
                                  WHERE child_subscription.finished = 0
                            )
              """
        }

        super(ExecuteDAOTest, self).setUp()

    def testGetSqlQuery(self):
        """
        Test the getsqlQuery method from the ExeccuteDAO class
        Tests the dictionary with all sql queries found in the DAO
        """
        self.sqlQueriesResult = self.daoObject.getSqlQuery()
        self.assertDictEqual(self.sqlQueriesResult, self.sqlQueriesExpected)

    def testDryRun(self):
        """
        Test execute in dryRun Mode
        """
        self.assertListEqual(self.daoObject(dryRun=True), [])


if __name__ == '__main__':
    unittest.main()



# Auto generated configuration file
# using:
# Revision: 1.392
# Source: /local/reps/CMSSW/CMSSW/Configuration/PyReleaseValidation/python/ConfigBuilder.py,v
# with command line options: step2 --datatier GEN-SIM-DIGI-RAW-HLTDEBUG --conditions auto:startup -s DIGI,L1,DIGI2RAW,HLT,RAW2DIGI,L1Reco --eventcontent FEVTDEBUGHLT --io DIGI.io --python DIGI.py -n 100 --no_exec --filein filelist:step1_dbsquery.log --fileout file:step2.root
# modified to reduce outside dependencies - this probably doesn't do anything useful anymore
# hopefully its still useful as a test

import FWCore.ParameterSet.Config as cms

process = cms.Process('HLT')

# import of standard configurations
#process.load('Configuration.StandardSequences.Services_cff')
#process.load('SimGeneral.HepPDTESSource.pythiapdt_cfi')
#process.load('FWCore.MessageService.MessageLogger_cfi')
#process.load('Configuration.EventContent.EventContent_cff')
#process.load('SimGeneral.MixingModule.mixNoPU_cfi')
#process.load('Configuration.StandardSequences.GeometryRecoDB_cff')
#process.load('Configuration.StandardSequences.MagneticField_38T_cff')
#process.load('Configuration.StandardSequences.Digi_cff')
#process.load('Configuration.StandardSequences.SimL1Emulator_cff')
#process.load('Configuration.StandardSequences.DigiToRaw_cff')
#process.load('HLTrigger.Configuration.HLT_GRun_cff')
#process.load('Configuration.StandardSequences.RawToDigi_cff')
#process.load('Configuration.StandardSequences.L1Reco_cff')
#process.load('Configuration.StandardSequences.EndOfProcess_cff')
#process.load('Configuration.StandardSequences.FrontierConditions_GlobalTag_cff')

process.maxEvents = cms.untracked.PSet(
    input = cms.untracked.int32(100)
)

# Input source
process.source = cms.Source("PoolSource",
    secondaryFileNames = cms.untracked.vstring(),
    fileNames = cms.untracked.vstring('/store/relval/CMSSW_6_0_0_pre3/RelValSingleElectronPt10/GEN-SIM/START60_V0-v1/0124/B64692A8-9C8E-E111-80E4-0018F3D0967E.root')
)

process.options = cms.untracked.PSet(

)

# Production Info
process.configurationMetadata = cms.untracked.PSet(
    version = cms.untracked.string('$Revision: 1.392 $'),
    annotation = cms.untracked.string('step2 nevts:100'),
    name = cms.untracked.string('PyReleaseValidation')
)

# Output definition

process.FEVTDEBUGHLToutput = cms.OutputModule("PoolOutputModule",
    splitLevel = cms.untracked.int32(0),
    eventAutoFlushCompressedSize = cms.untracked.int32(5242880),
#    outputCommands = process.FEVTDEBUGHLTEventContent.outputCommands,
    fileName = cms.untracked.string('file:step2.root'),
    dataset = cms.untracked.PSet(
        filterName = cms.untracked.string(''),
        dataTier = cms.untracked.string('GEN-SIM-DIGI-RAW-HLTDEBUG')
    )
)

# Additional output definition

# Other statements
#process.GlobalTag.globaltag = 'START60_V4::All'

# Path and EndPath definitions
process.digitisation_step = cms.Path() #cms.Path(process.pdigi)
process.L1simulation_step = cms.Path() #cms.Path(process.SimL1Emulator)
process.digi2raw_step = cms.Path() #cms.Path(process.DigiToRaw)
process.raw2digi_step = cms.Path() #cms.Path(process.RawToDigi)
process.L1Reco_step = cms.Path() #cms.Path(process.L1Reco)
process.endjob_step = cms.Path() #cms.EndPath(process.endOfProcess)
process.FEVTDEBUGHLToutput_step = cms.EndPath() #cms.EndPath(process.FEVTDEBUGHLToutput)

# Schedule definition
process.schedule = cms.Schedule(process.digitisation_step,process.L1simulation_step,process.digi2raw_step)
#process.schedule.extend(process.HLTSchedule)
process.schedule.extend([process.raw2digi_step,process.L1Reco_step,process.endjob_step,process.FEVTDEBUGHLToutput_step])

# customisation of the process.

# Automatic addition of the customisation function from HLTrigger.Configuration.customizeHLTforMC
#from HLTrigger.Configuration.customizeHLTforMC import customizeHLTforMC

#call to customisation function customizeHLTforMC imported from HLTrigger.Configuration.customizeHLTforMC
#process = customizeHLTforMC(process)

# End of customisation functions



#!/usr/bin/python

"""
_BasePlugin_t_

BasePlugin unittests
"""

from __future__ import division

import unittest

from WMCore_t.BossAir_t.BossAir_t import BossAirTest

from WMCore.BossAir.Plugins.BasePlugin import BasePlugin, BossAirPluginException


class BasePluginTest(BossAirTest):
    """
    _BasePluginTest_

    Tests for BasePlugin
    """

    def testScramArchToOS(self):
        """
        _testScramArchToOS_

        Test the conversion of the ScramArch string to requiredOS
        """
        bp = BasePlugin(config=None)

        self.assertEqual(bp.scramArchtoRequiredOS('slc5_blah_blah'), 'rhel6')
        self.assertEqual(bp.scramArchtoRequiredOS('slc6_blah_blah'), 'rhel6')
        self.assertEqual(bp.scramArchtoRequiredOS('slc7_blah_blah'), 'rhel7')
        self.assertEqual(bp.scramArchtoRequiredOS('cc8_blah_blah'), 'rhel8')
        self.assertEqual(bp.scramArchtoRequiredOS('cs8_blah_blah'), 'rhel8')
        self.assertEqual(bp.scramArchtoRequiredOS('alma8_blah_blah'), 'rhel8')
        self.assertEqual(bp.scramArchtoRequiredOS('el8_blah_blah'), 'rhel8')

        self.assertEqual(bp.scramArchtoRequiredOS(None), 'any')
        self.assertEqual(bp.scramArchtoRequiredOS(""), 'any')
        self.assertEqual(bp.scramArchtoRequiredOS([]), 'any')

        self.assertEqual(bp.scramArchtoRequiredOS(['slc6_blah_blah', 'slc7_blah_blah']), 'rhel6,rhel7')
        self.assertEqual(bp.scramArchtoRequiredOS(['slc6_blah_blah', 'alma8_blah_blah']), 'rhel6,rhel8')

        # unexpected case, a ScramArch being requested without the map implemented
        self.assertEqual(bp.scramArchtoRequiredOS('slc1_blah_blah'), '')
        return

    def testScramArchtoRequiredArch(self):
        """
        Test mapping of ScramArch to a given architecture
        """
        bp = BasePlugin(config=None)

        self.assertEqual(bp.scramArchtoRequiredArch('slc5_amd64_gcc481'), 'X86_64')
        self.assertEqual(bp.scramArchtoRequiredArch('slc6_amd64_gcc630'), 'X86_64')
        self.assertEqual(bp.scramArchtoRequiredArch('slc7_amd64_gcc10'), 'X86_64')
        self.assertEqual(bp.scramArchtoRequiredArch('slc7_aarch64_gcc700'), 'aarch64')
        self.assertEqual(bp.scramArchtoRequiredArch('slc7_ppc64le_gcc9'), 'ppc64le')
        self.assertIsNone(bp.scramArchtoRequiredArch(None))
        self.assertIsNone(bp.scramArchtoRequiredArch(None))
        with self.assertRaises(BossAirPluginException):
            bp.scramArchtoRequiredArch("slc7_BLAH_gcc700")

        self.assertEqual(bp.scramArchtoRequiredArch(['slc5_amd64_gcc481', 'slc6_amd64_gcc630']), 'X86_64')
        test = (bp.scramArchtoRequiredArch(['slc7_amd64_gcc10', 'slc7_aarch64_gcc700'])).split(',')
        test.sort()
        self.assertEqual(test, ['X86_64','aarch64'])
        test = (bp.scramArchtoRequiredArch(['slc7_amd64_gcc10', 'slc7_aarch64_gcc700', 'slc7_ppc64le_gcc9'])).split(',')
        test.sort()
        self.assertEqual(test, ['X86_64','aarch64','ppc64le'])
        test = (bp.scramArchtoRequiredArch(['slc7_aarch64_gcc700', 'slc7_ppc64le_gcc9'])).split(',')
        test.sort()
        self.assertEqual(test, ['aarch64', 'ppc64le'])

        return

if __name__ == '__main__':
    unittest.main()



#!/usr/bin/env python
"""
_BasePlugin_

Base class for BossAir plugins
"""

from builtins import object, str, bytes
from future.utils import viewvalues

from Utils.Utilities import decodeBytesToUnicode
from WMCore.WMException import WMException
from WMCore.WMRuntime.Tools.Scram import ARCH_TO_OS, SCRAM_TO_ARCH



class BossAirPluginException(WMException):
    """

    Generalized exception class for BossAir Plugins


    """




class BasePlugin(object):
    """
    Base class for BossAir Plugins

    Does nothing useful, just to be overridden later
    """


    # the common states which needs to be mapped from each plugIn states
    globalState = ['Pending', 'Running', 'Complete', 'Error']

    @staticmethod
    def verifyState(stateMap):
        """
        Verify state of map values
        """
        for state in viewvalues(stateMap):
            if state not in BasePlugin.globalState:
                raise BossAirPluginException("not valid state %s" % state)
        return stateMap

    @staticmethod
    def stateMap():
        """
        Empty state map to allow instantiation of base class
        """
        stateDict = {}

        return stateDict

    def __init__(self, config):

        self.config = config

        # NOTE: Don't overwrite this.
        # However stateMap should be implemented in child class.
        self.states = list(self.stateMap())



    def submit(self, jobs, info=None):
        """
        _submit_

        Submits jobs
        """

        pass


    def track(self, jobs):
        """
        _track_

        Tracks jobs
        Returns three lists:
        1) the running jobs
        2) the jobs that need to be updated in the DB
        3) the complete jobs
        """

        return jobs, jobs, []


    def complete(self, jobs):
        """
        _complete_

        Run any complete code
        """


        pass


    def kill(self, jobs, raiseEx):
        """
        _kill_

        Kill any and all jobs
        """

        pass

    def updateJobInformation(self, workflow, **kwargs):
        """
        _updateJobInformation_

        Update information on pending jobs
        where supported, the values are updated
        are passed through the kwargs
        """
        pass

    def updateSiteInformation(self, jobs, siteName, excludeSite):
        """
        _updateSiteInformation_

        Update Site Information
        """
        pass

    @staticmethod
    def scramArchtoRequiredOS(scramArch=None):
        """
        Matches a ScramArch - or a list of it - against a map of Scram
        to Operating System

        :param scramArch: string or list of scramArches defined for a given job
        :return: a string with the required OS to use
        """
        defaultValue = 'any'
        if not scramArch:
            return defaultValue

        requiredOSes = set()
        if isinstance(scramArch, (str, bytes)):
            scramArch = [scramArch]
        elif not isinstance(scramArch, (list, tuple)):
            return defaultValue

        for validArch in scramArch:
            scramOS = validArch.split("_")[0]
            requiredOSes.update(ARCH_TO_OS.get(scramOS, []))

        return ','.join(sorted(requiredOSes))

    @staticmethod
    def scramArchtoRequiredArch(scramArch=None):
        """
        Converts a given ScramArch to a list of target CPU architectures.
        In case no scramArch is defined, leave the architecture undefined.
        :param scramArch: can be either a string or a list of ScramArchs
        :return: a string with the matched architecture
        """
        defaultArch = "X86_64"
        requiredArchs = set()
        if scramArch is None:
            return None
        elif isinstance(scramArch, (str, bytes)):
            scramArch = [scramArch]

        for item in scramArch:
            item = decodeBytesToUnicode(item)
            arch = item.split("_")[1]
            if arch not in SCRAM_TO_ARCH:
                msg = "Job configured to a ScramArch: '{}' not supported in BossAir".format(item)
                raise BossAirPluginException(msg)
            requiredArchs.add(SCRAM_TO_ARCH.get(arch))

        # now we have the final list of architectures
        archs = ",".join(requiredArchs)
        if archs == '':
            archs = defaultArch

        return archs



#!/usr/bin/env python
"""
_Couchapp_


This is a general unittest for me should I need to create and
test couchapps.  Its utility is probably limited for people who
know what they're doing, and for couchapps that will never need
to be altered
"""
import os
import threading
import unittest

from WMCore_t.WMSpec_t.TestSpec import testWorkload

import WMCore.WMBase
from WMCore.DataStructs.Run import Run
from WMCore.Database.CMSCouch import CouchServer
from WMCore.FwkJobReport.Report import Report
from WMCore.JobStateMachine.ChangeState import ChangeState
from WMCore.Services.UUIDLib import makeUUID
from WMCore.WMBS.File import File
from WMCore.WMBS.Fileset import Fileset
from WMCore.WMBS.Job import Job
from WMCore.WMBS.JobGroup import JobGroup
from WMCore.WMBS.Subscription import Subscription
from WMCore.WMBS.Workflow import Workflow
from WMCore.WMSpec.Makers.TaskMaker import TaskMaker
from WMQuality.TestInitCouchApp import TestInitCouchApp as TestInit


class CouchappTest(unittest.TestCase):

    def setUp(self):
        myThread = threading.currentThread()

        self.testInit = TestInit(__file__)
        self.testInit.setLogging()
        self.testInit.setDatabaseConnection()
        self.testInit.setSchema(customModules=["WMCore.WMBS"],
                                useDefault=False)
        self.databaseName = "couchapp_t_0"

        # Setup config for couch connections
        config = self.testInit.getConfiguration()

        self.testInit.setupCouch(self.databaseName, "WorkloadSummary")
        self.testInit.setupCouch("%s/jobs" % config.JobStateMachine.couchDBName, "JobDump")
        self.testInit.setupCouch("%s/fwjrs" % config.JobStateMachine.couchDBName, "FWJRDump")
        self.testInit.setupCouch(config.JobStateMachine.summaryStatsDBName, "SummaryStats")

        # Create couch server and connect to databases
        self.couchdb = CouchServer(config.JobStateMachine.couchurl)
        self.jobsdatabase = self.couchdb.connectDatabase("%s/jobs" % config.JobStateMachine.couchDBName)
        self.fwjrdatabase = self.couchdb.connectDatabase("%s/fwjrs" % config.JobStateMachine.couchDBName)
        self.statsumdatabase = self.couchdb.connectDatabase(config.JobStateMachine.summaryStatsDBName)

        # Create changeState
        self.changeState = ChangeState(config)
        self.config = config

        # Create testDir
        self.testDir = self.testInit.generateWorkDir()

        return

    def tearDown(self):

        self.testInit.clearDatabase(modules=["WMCore.WMBS"])
        self.testInit.tearDownCouch()
        self.testInit.delWorkDir()
        # self.testInit.tearDownCouch()
        return

    def createWorkload(self, workloadName='Test', emulator=True):
        """
        _createTestWorkload_

        Creates a test workload for us to run on, hold the basic necessities.
        """

        workload = testWorkload("Tier1ReReco")

        taskMaker = TaskMaker(workload, os.path.join(self.testDir, 'workloadTest'))
        taskMaker.skipSubscription = True
        taskMaker.processWorkload()

        workload.save(workloadName)

        return workload

    def createTestJobGroup(self, name="TestWorkthrough",
                           specLocation="spec.xml", error=False,
                           task="/TestWorkload/ReReco", nJobs=10):
        """
        _createTestJobGroup_

        Generate a test WMBS JobGroup with real FWJRs
        """

        myThread = threading.currentThread()

        testWorkflow = Workflow(spec=specLocation, owner="Simon",
                                name=name, task=task)
        testWorkflow.create()

        testWMBSFileset = Fileset(name=name)
        testWMBSFileset.create()

        testFileA = File(lfn=makeUUID(), size=1024, events=10)
        testFileA.addRun(Run(10, *[12312]))
        testFileA.setLocation('malpaquet')

        testFileB = File(lfn=makeUUID(), size=1024, events=10)
        testFileB.addRun(Run(10, *[12312]))
        testFileB.setLocation('malpaquet')

        testFileA.create()
        testFileB.create()

        testWMBSFileset.addFile(testFileA)
        testWMBSFileset.addFile(testFileB)
        testWMBSFileset.commit()
        testWMBSFileset.markOpen(0)

        testSubscription = Subscription(fileset=testWMBSFileset,
                                        workflow=testWorkflow)
        testSubscription.create()

        testJobGroup = JobGroup(subscription=testSubscription)
        testJobGroup.create()

        for i in range(0, nJobs):
            testJob = Job(name=makeUUID())
            testJob.addFile(testFileA)
            testJob.addFile(testFileB)
            testJob['retry_count'] = 1
            testJob['retry_max'] = 10
            testJob['mask'].addRunAndLumis(run=10, lumis=[12312, 12313])
            testJobGroup.add(testJob)

        testJobGroup.commit()

        report = Report()
        if error:
            path = os.path.join(WMCore.WMBase.getTestBase(),
                                "WMComponent_t/JobAccountant_t/fwjrs", "badBackfillJobReport.pkl")
        else:
            path = os.path.join(WMCore.WMBase.getTestBase(),
                                "WMComponent_t/JobAccountant_t/fwjrs", "PerformanceReport2.pkl")
        report.load(filename=path)

        self.changeState.propagate(testJobGroup.jobs, 'created', 'new')
        self.changeState.propagate(testJobGroup.jobs, 'executing', 'created')
        self.changeState.propagate(testJobGroup.jobs, 'complete', 'executing')
        for job in testJobGroup.jobs:
            job['fwjr'] = report
        self.changeState.propagate(testJobGroup.jobs, 'jobfailed', 'complete')
        self.changeState.propagate(testJobGroup.jobs, 'retrydone', 'jobfailed')
        self.changeState.propagate(testJobGroup.jobs, 'exhausted', 'retrydone')
        self.changeState.propagate(testJobGroup.jobs, 'cleanout', 'exhausted')

        testSubscription.completeFiles([testFileA, testFileB])

        return testJobGroup


if __name__ == '__main__':
    unittest.main()



"""
File       : MSPileupData.py
Author     : Valentin Kuznetsov <vkuznet AT gmail dot com>
Description: Unit tests for MicorService/MSPileup/MSPileupData.py module
"""

# system modules
import time
import unittest

# WMCore modules
from WMCore.MicroService.MSPileup.MSPileupData import MSPileupData, stripKeys, getNewTimestamp, \
    customDID, addTransitionRecord
from WMCore.MicroService.MSPileup.MSPileupError import MSPILEUP_SCHEMA_ERROR
from WMCore.MicroService.Tools.Common import getMSLogger
from Utils.Timers import encodeTimestamp, decodeTimestamp, gmtimeSeconds


class MSPileupTest(unittest.TestCase):
    """Unit test for MSPileupData module"""

    def setUp(self):
        """setup unit test class"""
        self.logger = getMSLogger(False)
        self.validRSEs = ['rse1']
        msConfig = {'reqmgr2Url': 'http://localhost',
                    'rucioAccount': 'wmcore_test',
                    'rucioUrl': 'http://cms-rucio-int.cern.ch',
                    'rucioAuthUrl': 'https://cms-rucio-auth-int.cern.ch',
                    'validRSEs': self.validRSEs,
                    'mongoDB': 'msPileupDB',
                    'mongoDBCollection': 'msPileupDBCollection',
                    'mongoDBServer': 'mongodb://localhost',
                    'mongoDBReplicaSet': '',
                    'mongoDBUser': None,
                    'mongoDBPassword': None,
                    'mockMongoDB': True}
        self.mgr = MSPileupData(msConfig, skipRucio=True)

        pname = '/skldjflksdjf/skldfjslkdjf/PREMIX'
        now = int(time.mktime(time.gmtime()))
        expectedRSEs = self.validRSEs
        fullReplicas = 1
        pileupSize = 1
        ruleIds = []
        campaigns = []
        containerFraction = 1.0
        replicationGrouping = "ALL"

        pdict = {
            'pileupName': pname,
            'pileupType': 'classic',
            'insertTime': now,
            'lastUpdateTime': now,
            'expectedRSEs': expectedRSEs,
            'currentRSEs': expectedRSEs,
            'fullReplicas': fullReplicas,
            'campaigns': campaigns,
            'containerFraction': containerFraction,
            'replicationGrouping': replicationGrouping,
            'activatedOn': now,
            'deactivatedOn': now,
            'active': True,
            'pileupSize': pileupSize,
            'customName': '',
            'transition': [],
            'ruleIds': ruleIds}
        self.doc = pdict
        self.pname = pname
        self.userDN = 'test-dn'
        self.tranRecord = {'containerFraction': 1.0,
                           'customDID': self.pname,
                           'DN': self.userDN,
                           'updateTime': gmtimeSeconds()}

    def testCustomDID(self):
        """Test the customDID function"""
        pname = "/abc/xyz/MINIAOD"
        did = customDID(pname)
        self.assertTrue(did == (pname + '-V1'))
        did = customDID(did)
        self.assertTrue(did == (pname + '-V2'))

        # test more complex suffix
        pname = "/abc/xyz/MINIAOD-V123"
        did = customDID(pname)
        expect = "/abc/xyz/MINIAOD-V124"
        self.assertTrue(did == expect)

        # test last digit wrap
        pname = "/abc/xyz/MINIAOD-V19"
        did = customDID(pname)
        expect = "/abc/xyz/MINIAOD-V20"
        self.assertTrue(did == expect)

        # test last digit wrap
        pname = "/abc/xyz/MINIAOD-V99"
        did = customDID(pname)
        expect = "/abc/xyz/MINIAOD-V100"
        self.assertTrue(did == expect)

    def testUpdateTransitionRecord(self):
        """Test the addTransitionRecord function"""
        self.logger.info("test addTransitionRecord function")
        doc = dict(self.doc)
        userDN = ''
        doc['transition'] = [self.tranRecord]
        addTransitionRecord(doc, userDN, self.logger)
        self.logger.info("1st test (empty DN) %s", doc)
        self.assertTrue(len(doc['transition']), 1)
        trec = doc['transition'][0]
        self.logger.info("trec %s", trec)
        self.assertTrue(trec['DN'], self.userDN)

        # test when our document contains smaller fraction
        fraction = 0.5
        doc['containerFraction'] = fraction
        doc['transition'] = [self.tranRecord]
        addTransitionRecord(doc, userDN, self.logger)
        self.logger.info("2nd test (container fraction) %s", doc)
        self.assertTrue(len(doc['transition']), 2)
        # check that all transition record have the same DN
        for trec in doc['transition']:
            self.assertTrue(trec['DN'], self.userDN)
        # check container fraction of last record
        trec = doc['transition'][-1]
        self.logger.info("trec %s", trec)
        self.assertTrue(trec['containerFraction'], fraction)


        # test use-case when we supply the same container fraction
        addTransitionRecord(doc, self.userDN, self.logger)
        self.logger.info("2nd test (same container fraction) %s", doc)
        self.assertTrue(len(doc['transition']), 2)

        # test when our document contains smaller fraction and we add new transition record with custom DN
        userDN = 'bla'
        addTransitionRecord(doc, userDN, self.logger)
        self.logger.info("3rd test (custom DN) %s", doc)
        self.assertTrue(len(doc['transition']), 3)
        # check that last transition record will have custom DN
        trec = doc['transition'][-1]
        self.assertTrue(trec['DN'], userDN)
        # and first and second transition record should have original userDN
        trec = doc['transition'][0]
        self.assertTrue(trec['DN'], self.userDN)
        trec = doc['transition'][1]
        self.assertTrue(trec['DN'], self.userDN)

        # test daemon use-case, i.e. when we update pileup record attribute w/o transition
        doc = dict(self.doc)
        doc['transition'] = [self.tranRecord]
        doc['pileupSize'] = 100
        userDN = ''
        addTransitionRecord(doc, userDN, self.logger)
        self.logger.info("4th test (daemon test) %s", doc)
        self.assertTrue(len(doc['transition']), 1)
        trec = doc['transition'][0]
        self.assertTrue(trec['DN'], self.userDN)

    def testStripKeys(self):
        """
        Unit test to test stripKeys helper function
        """
        skeys = ['_id']
        expect = {'pileupId': 1}
        pdict = {'pileupId': 1, '_id': 1}
        pdict = stripKeys(pdict, skeys)
        self.assertDictEqual(pdict, expect)

        pdict = {'pileupId': 1, '_id': 1}
        results = [pdict]
        results = stripKeys(results, skeys)
        self.assertDictEqual(pdict, expect)

    def testTimestampsSerialization(self):
        """
        Unit test to test serialization of timestamps
        """
        tkeys = ['insertTime', 'lastUpdateTime']
        doc = {'pileupId': 1}
        now = int(time.time())
        gnow = time.gmtime(now)
        expect = time.strftime("%Y-%m-%dT%H:%M:%SZ", gnow)
        for key in tkeys:
            doc.update({key: now})
        # encode time stamps
        for key in ['insertTime', 'lastUpdateTime']:
            doc[key] = encodeTimestamp(doc[key])
            self.assertEqual(doc[key], expect)
        # decode time stamps
        for key in ['insertTime', 'lastUpdateTime']:
            doc[key] = decodeTimestamp(doc[key])
            self.assertEqual(int(doc[key]), now)

    def testDocs(self):
        """
        Unit test to test document creation and storage in MongoDB
        """
        skeys = ['_id']
        pdict = dict(self.doc)
        pname = self.pname
        out = self.mgr.createPileup(pdict, self.validRSEs, userDN=self.userDN)
        self.assertEqual(len(out), 0)

        # now fail the RSE validation
        out = self.mgr.createPileup(pdict, ['rse2'], userDN=self.userDN)[0]
        self.assertEqual(out['error'], "MSPileupError")
        self.assertEqual(out['code'], 7)
        expect = "Failed to create MSPileupObj, MSPileup input is invalid, expectedRSEs value ['rse1'] is not in validRSEs ['rse2']"
        self.assertEqual(out['message'], expect)

        spec = {'pileupName': pname}
        results = self.mgr.getPileup(spec)
        doc = results[0]
        doc = stripKeys(doc, skeys)
        # add transition record to original pdict document because it is added internally in createPileup API
        pdict['transition'] = doc['transition']
        self.assertDictEqual(pdict, doc)

        doc.update({'pileupSize': 2})
        out = self.mgr.updatePileup(doc, self.validRSEs)
        self.assertEqual(len(out), 0)
        results = self.mgr.getPileup(spec)
        doc = results[0]
        doc = stripKeys(doc, skeys)
        self.assertEqual(doc['pileupSize'], 2)

        self.mgr.deletePileup(spec)
        self.assertEqual(len(out), 0)
        results = self.mgr.getPileup(spec)
        self.assertEqual(len(results), 0)

    def testMSPileupQuery(self):
        "test MSPileup query API"
        pname = '/skldjflksdjf/skldfjslkdjf/PREMIX'
        spec = {"pileupName": pname}
        res = self.mgr.getPileup(spec)
        self.assertEqual(len(res), 0)

        spec = {"bla": 1}
        res = self.mgr.sanitizeQuery(spec)
        # validate should return list of single dictionary
        # [{'data': {'bla': 1}, 'message': 'schema error', 'code': 7, 'error': 'MSPileupError'}]
        self.assertEqual(len(res), 1)
        err = res[0]
        self.assertEqual(err.get('error'), 'MSPileupError')
        self.assertEqual(err.get('code'), MSPILEUP_SCHEMA_ERROR)

        # and we should get zero results
        res = self.mgr.getPileup(spec)
        self.assertEqual(len(res), 0)

    def testGetNewTimestamp(self):
        """Test the getNewTimestamp function"""
        timeNow = gmtimeSeconds()
        resp = getNewTimestamp({})
        self.assertEqual(len(resp), 1)
        self.assertTrue(resp['lastUpdateTime'] >= timeNow)

        resp = getNewTimestamp({'lastUpdateTime': 1})
        self.assertEqual(len(resp), 1)
        self.assertTrue(resp['lastUpdateTime'] >= timeNow)

        resp = getNewTimestamp({'active': True})
        self.assertEqual(len(resp), 2)
        self.assertTrue(resp['lastUpdateTime'] >= timeNow)
        self.assertTrue(resp['activatedOn'] >= timeNow)
        self.assertFalse('deactivatedOn' in resp)

        resp = getNewTimestamp({'active': False})
        self.assertEqual(len(resp), 2)
        self.assertTrue(resp['lastUpdateTime'] >= timeNow)
        self.assertTrue(resp['deactivatedOn'] >= timeNow)
        self.assertFalse('activatedOn' in resp)


if __name__ == '__main__':
    unittest.main()



#!/usr/bin/env python
"""
Locations_t

Unit tests for the Locations DAO objects.
"""

import threading
import time
import unittest

from WMCore.DAOFactory import DAOFactory
from WMCore.Database.Transaction import Transaction
from WMQuality.TestInit import TestInit


class LocationsTest(unittest.TestCase):
    def setUp(self):
        """
        _setUp_

        Setup the database and logging connection.  Try to create all of the
        WMBS tables.
        """
        self.testInit = TestInit(__file__)
        self.testInit.setLogging()
        self.testInit.setDatabaseConnection()
        self.testInit.setSchema(customModules=["WMCore.WMBS"],
                                useDefault=False)
        return

    def tearDown(self):
        """
        _tearDown_

        Drop all the WMBS tables.
        """
        self.testInit.clearDatabase()
        return

    def testCreateDeleteList(self):
        """
        _testCreateDeleteList_

        Test the creation, listing and deletion of locations in WMBS.
        """
        goldenLocations = ["goodse.cern.ch", "goodse.fnal.gov"]

        myThread = threading.currentThread()
        daoFactory = DAOFactory(package="WMCore.WMBS", logger=myThread.logger,
                                dbinterface=myThread.dbi)

        locationNew = daoFactory(classname="Locations.New")
        for location in goldenLocations:
            # The following is intentional, I want to test that inserting the
            # same location multiple times does not cause problems.
            locationNew.execute(siteName=location, runningSlots=300, pendingSlots=150)
            locationNew.execute(siteName=location, runningSlots=300, pendingSlots=150)

        locationNew.execute(siteName="empty_site")
        goldenLocations.append("empty_site")

        locationList = daoFactory(classname="Locations.List")
        currentLocations = locationList.execute()
        for location in currentLocations:
            assert location[1] in goldenLocations, \
                "ERROR: Unknown location was returned"

            if location[1] == "empty_site":
                assert location[2] == 0, \
                    "ERROR: Site has wrong number of pending slots."
                self.assertEqual(location[3], 0,
                                 "ERROR: Site has wrong number of running slots")
            else:
                assert location[2] == 150, \
                    "ERROR: Site has wrong number of pending slots."
                self.assertEqual(location[3], 300,
                                 "ERROR: Site has wrong number of running slots")

            goldenLocations.remove(location[1])

        assert len(goldenLocations) == 0, \
            "ERROR: Some locations are missing..."

        locationDelete = daoFactory(classname="Locations.Delete")
        locationDelete.execute(siteName="goodse.fnal.gov")
        locationDelete.execute(siteName="goodse.cern.ch")

        currentLocations = locationList.execute()
        assert len(currentLocations) == 1, \
            "ERROR: Not all locations were deleted"
        assert currentLocations[0][1] == "empty_site", \
            "ERROR: The wrong sites were deleted."

        return

    def testListSites(self):
        """
        _testListSites

        Test the ability to list all sites in the database.
        """
        myThread = threading.currentThread()
        daoFactory = DAOFactory(package="WMCore.WMBS", logger=myThread.logger,
                                dbinterface=myThread.dbi)

        locationNew = daoFactory(classname="Locations.New")

        locationNew.execute("Satsuma")
        locationNew.execute("Choshu")
        locationNew.execute("Tosa")

        listSites = daoFactory(classname="Locations.ListSites")
        sites = listSites.execute()

        self.assertEqual("Satsuma" in sites, True)
        self.assertEqual("Choshu" in sites, True)
        self.assertEqual("Tosa" in sites, True)

    def testListSitesTransaction(self):
        """
        _testListSitesTransaction_

        Verify that select behave appropriately when dealing with transactions.
        """
        myThread = threading.currentThread()
        daoFactory = DAOFactory(package="WMCore.WMBS", logger=myThread.logger,
                                dbinterface=myThread.dbi)

        myThread.transaction.begin()

        localTransaction = Transaction(myThread.dbi)
        localTransaction.begin()

        locationNew = daoFactory(classname="Locations.New")

        locationNew.execute("Satsuma", conn=myThread.transaction.conn, transaction=True)
        locationNew.execute("Choshu", conn=myThread.transaction.conn, transaction=True)

        listSites = daoFactory(classname="Locations.ListSites")
        nonTransSites = listSites.execute(conn=localTransaction.conn, transaction=True)
        transSites = listSites.execute(conn=myThread.transaction.conn, transaction=True)

        assert len(nonTransSites) == 0, \
            "Error: Wrong number of sites in non transaction list."
        assert len(transSites) == 2, \
            "Error: Wrong number of sites in transaction list."
        assert "Satsuma" in transSites, \
            "Error: Site missing in transaction list."
        assert "Choshu" in transSites, \
            "Error: Site missing in transaction list."

        localTransaction.commit()
        myThread.transaction.commit()
        return

    def testJobSlots(self):
        """
        _testJobSlots_

        Test our ability to set/get pending and running slots
        """

        myThread = threading.currentThread()
        daoFactory = DAOFactory(package="WMCore.WMBS", logger=myThread.logger,
                                dbinterface=myThread.dbi)

        locationNew = daoFactory(classname="Locations.New")

        locationNew.execute("Satsuma")
        locationNew.execute("Choshu")
        locationNew.execute("Tosa")

        setPendingSlots = daoFactory(classname="Locations.SetPendingSlots")
        setPendingSlots.execute(siteName='Satsuma', jobSlots=1868)
        setPendingSlots.execute(siteName='Choshu', jobSlots=1868)
        setPendingSlots.execute(siteName='Tosa', jobSlots=1868)

        setRunningSlots = daoFactory(classname="Locations.SetRunningSlots")
        setRunningSlots.execute(siteName='Satsuma', jobSlots=42)
        setRunningSlots.execute(siteName='Choshu', jobSlots=42)
        setRunningSlots.execute(siteName='Tosa', jobSlots=42)

        locationList = daoFactory(classname="Locations.List")
        currentLocations = locationList.execute()

        for location in currentLocations:
            self.assertEqual(location[2], 1868)
            self.assertEqual(location[3], 42)

        jobSlotDAO = daoFactory(classname="Locations.GetJobSlots")
        jobSlots = jobSlotDAO.execute("Satsuma")
        self.assertEqual(jobSlots, [{'running_slots': 42, 'pending_slots': 1868}])

        pendingSlotDAO = daoFactory(classname="Locations.GetPendingSlots")
        pendingSlots = pendingSlotDAO.execute("Satsuma")
        self.assertEqual(pendingSlots, 1868)

        runningSlotDAO = daoFactory(classname="Locations.GetRunningSlots")
        runningSlots = runningSlotDAO.execute("Satsuma")
        self.assertEqual(runningSlots, 42)

        return

    def testGetSiteInfo(self):
        """
        _testGetSiteInfo_

        Test our ability to retrieve ce, se names, etc.
        """

        myThread = threading.currentThread()
        daoFactory = DAOFactory(package="WMCore.WMBS", logger=myThread.logger,
                                dbinterface=myThread.dbi)

        locationNew = daoFactory(classname="Locations.New")

        locationNew.execute(siteName="Satsuma", ceName="Satsuma", pnn="Satsuma", runningSlots=50, pendingSlots=20)
        locationNew.execute(siteName="Choshu", ceName="Choshu", pnn="Choshu", runningSlots=50, pendingSlots=20)
        locationNew.execute(siteName="Tosa", ceName="Tosa", pnn="Choshu", runningSlots=50, pendingSlots=20)

        locationInfo = daoFactory(classname="Locations.GetSiteInfo")

        result = locationInfo.execute(siteName="Choshu")

        self.assertEqual(result[0]['ce_name'], 'Choshu')
        self.assertEqual(result[0]['pnn'], ['Choshu'])
        self.assertEqual(result[0]['site_name'], 'Choshu')
        self.assertEqual(result[0]['pending_slots'], 20)
        self.assertEqual(result[0]['running_slots'], 50)

        return

    def testDrain(self):
        """Drain sites"""
        myThread = threading.currentThread()
        daoFactory = DAOFactory(package="WMCore.WMBS", logger=myThread.logger,
                                dbinterface=myThread.dbi)
        locationNew = daoFactory(classname="Locations.New")
        locationInfo = daoFactory(classname="Locations.GetSiteInfo")
        setState = daoFactory(classname="Locations.SetState")

        # add site and check it's in normal
        locationNew.execute(siteName="Satsuma", ceName="Satsuma", pnn="Satsuma",
                            stateTime=int(time.time()))
        result = locationInfo.execute(siteName="Satsuma")
        self.assertEqual(result[0]['site_name'], 'Satsuma')
        self.assertEqual(result[0]['state'], 'Normal')

        # Now set it to down, it's dead jim
        setState.execute(siteName='Satsuma', state='Down', stateTime=int(time.time()))
        result = locationInfo.execute(siteName="Satsuma")
        self.assertEqual(result[0]['site_name'], 'Satsuma')
        self.assertEqual(result[0]['state'], 'Down')

        # Now let's set it to aborted mode
        setState.execute(siteName='Satsuma', state='Aborted', stateTime=int(time.time()))
        result = locationInfo.execute(siteName="Satsuma")
        self.assertEqual(result[0]['site_name'], 'Satsuma')
        self.assertEqual(result[0]['state'], 'Aborted')

        # Now let's set it to drain mode
        setState.execute(siteName='Satsuma', state='Draining', stateTime=int(time.time()))
        result = locationInfo.execute(siteName="Satsuma")
        self.assertEqual(result[0]['site_name'], 'Satsuma')
        self.assertEqual(result[0]['state'], 'Draining')


if __name__ == "__main__":
    unittest.main()



#!/usr/bin/env python
"""
_WMConfigCache_t_

Test class for the WMConfigCache
"""

from future import standard_library
standard_library.install_aliases()
import urllib.parse

import os
import unittest
import tempfile
import subprocess

from WMCore.Agent.Configuration import Configuration
from WMCore.Cache.WMConfigCache import ConfigCache, ConfigCacheException
from WMCore.WMBase import getTestBase
from WMQuality.TestInitCouchApp import TestInitCouchApp

class testWMConfigCache(unittest.TestCase):
    """
    _testWMConfigCache_

    Basic test class for configCache
    """
    def setUp(self):
        """
        _setUp_

        """
        self.testInit = TestInitCouchApp(__file__)
        self.testInit.setLogging()
        self.testInit.setupCouch("config_test", "GroupUser", "ConfigCache")

        self.testDir = self.testInit.generateWorkDir()
        return

    def tearDown(self):
        """
        _tearDown_

        Clear out the database.
        """
        self.testInit.delWorkDir()
        self.testInit.tearDownCouch()
        return

    def testA_basicConfig(self):
        """
        _basicConfig_

        Basic configCache stuff.
        """
        PSetTweak = "Hello, I am a PSetTweak.  It's nice to meet you."

        configCache = ConfigCache(os.environ["COUCHURL"], couchDBName = 'config_test')
        configCache.createUserGroup(groupname = "testGroup", username = 'testOps')
        configCache.setPSetTweaks(PSetTweak = PSetTweak)
        configCache.save()

        configCache2 = ConfigCache(os.environ["COUCHURL"], couchDBName = 'config_test',
                                   id = configCache.getCouchID(),
                                   rev = configCache.getCouchRev())
        configCache2.loadByID(configCache.getCouchID())

        self.assertEqual(configCache2.getPSetTweaks(), PSetTweak)

        configCache2.delete()
        configCache3 = ConfigCache(os.environ["COUCHURL"], couchDBName = 'config_test',
                                   id = configCache.getCouchID(),
                                   rev = configCache.getCouchRev())

        testFlag = False

        # It should fail to load deleted documents
        try:
            configCache3.loadByID(configCache.getCouchID())
        except ConfigCacheException:
            testFlag = True

        self.assertTrue(testFlag)

        return

    def testB_addingConfigsAndTweaks(self):
        """
        _addingConfigsAndTweaks_

        Test adding config files and tweak files
        """
        PSetTweak = "Hello, I am a PSetTweak.  It's nice to meet you."
        attach    = "Hello, I am an attachment"

        configCache = ConfigCache(os.environ["COUCHURL"], couchDBName = 'config_test')
        configCache.createUserGroup(groupname = "testGroup", username = 'testOps')
        configCache.setPSetTweaks(PSetTweak = PSetTweak)
        configCache.attachments['attach1'] = attach
        psetPath = os.path.join(getTestBase(), "WMCore_t/Cache_t/PSet.txt")
        psetPath = "file://" + urllib.parse.quote(os.path.abspath(psetPath))
        configCache.addConfig(newConfig = psetPath, psetHash = None)

        configCache.setLabel("sample-label")
        configCache.setDescription("describe this config here")
        configCache.save()
        configString1 = configCache.getConfig()

        configCache2 = ConfigCache(os.environ["COUCHURL"], couchDBName = 'config_test',
                                   id = configCache.getCouchID(),
                                   rev = configCache.getCouchRev())
        configCache2.loadByID(configCache.getCouchID())
        configString2 = configCache2.getConfig()

        self.assertEqual(configString1, configString2)
        self.assertEqual(configCache2.attachments.get('attach1', None), attach)

        configCache.delete()
        return


    def testC_testViews(self):
        """
        _testViews_

        Prototype test for what should be a lot of other tests.
        """
        PSetTweak = "Hello, I am a PSetTweak.  It's nice to meet you."
        attach    = "Hello, I am an attachment"

        configCache = ConfigCache(os.environ["COUCHURL"], couchDBName = 'config_test')
        configCache.createUserGroup(groupname = "testGroup", username = 'testOps')
        configCache.setPSetTweaks(PSetTweak = PSetTweak)
        configCache.attachments['attach1'] = attach
        configCache.document['md5_hash'] = "somemd5"
        psetPath = os.path.join(getTestBase(), "WMCore_t/Cache_t/PSet.txt")
        psetPath = "file://" + urllib.parse.quote(os.path.abspath(psetPath))
        configCache.addConfig(newConfig = psetPath, psetHash = None)
        configCache.save()

        configCache2 = ConfigCache(os.environ["COUCHURL"], couchDBName = 'config_test')
        configCache2.document['md5_hash'] = configCache.document['md5_hash']
        configCache2.load()

        self.assertEqual(configCache2.attachments.get('attach1', None), attach)
        configCache2.delete()
        return

    def testD_LoadConfigCache(self):
        """
        _LoadConfigCache_

        Actually load the config cache using plain .load()
        Tests to make sure that if we pass in an id field it gets used to load configs
        """

        configCache = ConfigCache(os.environ["COUCHURL"], couchDBName = 'config_test')
        configCache.createUserGroup(groupname = "testGroup", username = 'testOps')
        configCache.setLabel("labelA")
        configCache.save()

        configCache2 = ConfigCache(os.environ["COUCHURL"], couchDBName = 'config_test',
                                   id = configCache.getCouchID(),
                                   rev = configCache.getCouchRev())
        configCache2.load()
        self.assertEqual(configCache2.document['owner'],
                         {'group': 'testGroup', 'user': 'testOps'})
        self.assertEqual(configCache2.document['description'],
                         {'config_desc': None, 'config_label': 'labelA'})
        return

    def testE_SaveConfigFileToDisk(self):
        """
        _SaveConfigFileToDisk_

        Check and see if we can save the config file attachment to disk
        """
        targetFile = os.path.join(self.testDir, 'configCache.test')

        configCache = ConfigCache(os.environ["COUCHURL"], couchDBName = 'config_test')
        configCache.createUserGroup(groupname = "testGroup", username = 'testOps')
        configCache.attachments['configFile'] = 'ThisIsAConfigFile'
        configCache.saveConfigToDisk(targetFile = targetFile)

        f = open(targetFile, 'r')
        content = f.read()
        f.close()

        self.assertEqual(content, configCache.getConfig())
        return

    def testListAllConfigs(self):
        """
        _testListAllConfigs_

        Verify that the list all configs method works correctly.
        """
        configCacheA = ConfigCache(os.environ["COUCHURL"], couchDBName = 'config_test')
        configCacheA.createUserGroup(groupname = "testGroup", username = 'testOps')
        configCacheA.setLabel("labelA")
        configCacheA.save()

        configCacheB = ConfigCache(os.environ["COUCHURL"], couchDBName = 'config_test')
        configCacheB.createUserGroup(groupname = "testGroup", username = 'testOps')
        configCacheB.setLabel("labelB")
        configCacheB.save()

        configs = configCacheA.listAllConfigsByLabel()

        self.assertEqual(len(configs), 2,
                         "Error: There should be two configs")
        self.assertEqual(configs["labelA"], configCacheA.getCouchID(),
                         "Error: Label A is wrong.")
        self.assertEqual(configs["labelB"], configCacheB.getCouchID(),
                         "Error: Label B is wrong.")
        return

if __name__ == "__main__":
    unittest.main()



import sys, os

extensions = ['sphinx.ext.autodoc', 'sphinx.ext.doctest',
              'sphinx.ext.intersphinx', 'sphinx.ext.todo',
              'sphinx.ext.ifconfig', 'sphinx.ext.inheritance_diagram',
              'sphinx.ext.viewcode']
templates_path = ['_templates']
source_suffix = '.rst'
source_encoding = 'utf-8'
master_doc = 'index'
project = 'CMS WMCore'
copyright = 'CERN and Fermilab'
version = '<VERSION>'
release = '<VERSION>'
today_fmt = '%B %d, %Y'
#unused_docs = []
#exclude_trees = []
#default_role = None
add_function_parentheses = True
add_module_names = False
show_authors = True
pygments_style = 'sphinx'
#modindex_common_prefix = []

autoclass_content = 'both'
autodoc_member_order = 'bysource'
autodoc_default_flags = ['members', 'undoc-members', 'private-members',
                         'special-members', 'show-inheritance']

html_theme = 'nature'
#html_theme_options = {}
#html_theme_path = []
html_title = "WMCore %s" % version
#html_short_title = None
#html_logo = None
#html_favicon = None
#html_static_path = ['_static']
html_last_updated_fmt = '%b %d, %Y'
#html_use_smartypants = True
#html_sidebars = {}
#html_additional_pages = {}
#html_use_modindex = True
#html_use_index = True
#html_split_index = False
html_show_sourcelink = False
#html_use_opensearch = ''
#html_file_suffix = ''
htmlhelp_basename = 'wmcdoc'

latex_paper_size = 'a4'
#latex_font_size = '10pt'
# (source start file, target name, title, author, documentclass [howto/manual]).
latex_documents = [
  ('index', 'WMCore.tex', u'WMCore Documentation',
   u'CMS DM/WM Project', 'manual'),
]

#latex_logo = None
#latex_use_parts = False
#latex_preamble = ''
#latex_appendices = []
#latex_use_modindex = True

# Example configuration for intersphinx: refer to the Python standard library.
intersphinx_mapping = {'http://docs.python.org/': None}



"""
_Tier0Plugin_t_

Test the Tier-0 plugin for the AnalyticsDataCollector

Created on Nov 7, 2012

@author: dballest
"""
from __future__ import division
from builtins import range

import os
import unittest

from WMComponent.AnalyticsDataCollector.Plugins.Tier0Plugin import Tier0Plugin
from WMCore.Services.RequestDB.RequestDBWriter import RequestDBWriter
from WMCore.WMBS.Fileset import Fileset
from WMCore.WMBS.Subscription import Subscription
from WMCore.WMBS.Workflow import Workflow
from WMCore.WMSpec.StdSpecs.PromptReco import PromptRecoWorkloadFactory
from WMCore.WMSpec.WMWorkload import newWorkload
from WMCore.WorkQueue.WMBSHelper import WMBSHelper
from WMQuality.TestInitCouchApp import TestInitCouchApp as TestInit


class Tier0PluginTest(unittest.TestCase):
    def setUp(self):
        """
        _setUp_

        Setup the test environment
        """
        self.testInit = TestInit(__file__)
        self.testInit.setDatabaseConnection()
        self.testInit.setSchema(["WMCore.WMBS"])
        self.requestCouchDB = 'wmstats_plugin_t'
        self.testInit.setupCouch(self.requestCouchDB, 'T0Request')
        self.testDir = self.testInit.generateWorkDir()
        reqDBURL = "%s/%s" % (os.environ['COUCHURL'], self.requestCouchDB)
        self.requestDBWriter = RequestDBWriter(reqDBURL, couchapp="T0Request")
        self.requestDBWriter._setNoStale()

        self.stateMap = {}
        self.orderedStates = []
        self.plugin = None

        return

    def tearDown(self):
        """
        _tearDown_

        Clear databases and delete files
        """
        self.testInit.tearDownCouch()
        self.testInit.clearDatabase()
        self.testInit.delWorkDir()

        return

    def setupRepackWorkflow(self):
        """
        _setupRepackWorkflow_

        Populate WMBS with a repack-like workflow,
        every subscription must be unfinished at first
        """

        workflowName = 'Repack_Run481516_StreamZ'
        mergeTasks = ['RepackMergewrite_QuadElectron_RAW', 'RepackMergewrite_TriPhoton_RAW',
                      'RepackMergewrite_SingleNeutrino_RAW']

        self.stateMap = {'Merge': [],
                         'Processing Done': []}
        self.orderedStates = ['Merge', 'Processing Done']

        # Populate WMStats
        self.requestDBWriter.insertGenericRequest({'RequestName': workflowName})
        self.requestDBWriter.updateRequestStatus(workflowName, 'Closed')

        # Create a wmspec in disk
        workload = newWorkload(workflowName)
        repackTask = workload.newTask('Repack')
        for task in mergeTasks:
            repackTask.addTask(task)
        repackTask.addTask('RepackCleanupUnmergedwrite_QuadElectron_RAW')

        specPath = os.path.join(self.testDir, 'Repack.pkl')
        workload.save(specPath)

        # Populate WMBS
        topFileset = Fileset(name='TestStreamerFileset')
        topFileset.create()

        options = {'spec': specPath, 'owner': 'ItsAMeMario',
                   'name': workflowName, 'wfType': 'tier0'}
        topLevelWorkflow = Workflow(task='/%s/Repack' % workflowName,
                                    **options)
        topLevelWorkflow.create()
        topLevelSub = Subscription(topFileset, topLevelWorkflow)
        topLevelSub.create()
        self.stateMap['Merge'].append(topFileset)
        for task in mergeTasks:
            mergeWorkflow = Workflow(task='/%s/Repack/%s' % (workflowName, task), **options)
            mergeWorkflow.create()
            unmergedFileset = Fileset(name='TestUnmergedFileset%s' % task)
            unmergedFileset.create()
            mergeSub = Subscription(unmergedFileset, mergeWorkflow)
            mergeSub.create()
            self.stateMap['Processing Done'].append(unmergedFileset)
        cleanupWorkflow = Workflow(task='/Repack_Run481516_StreamZ/Repack/RepackCleanupUnmergedwrite_QuadElectron_RAW',
                                   **options)
        cleanupWorkflow.create()
        unmergedFileset = Fileset(name='TestUnmergedFilesetToCleanup')
        unmergedFileset.create()
        cleanupSub = Subscription(unmergedFileset, cleanupWorkflow)
        cleanupSub.create()

        return

    def setupExpressWorkflow(self):
        """
        _setupExpressWorkflow_

        Populate WMBS with a express-like workflow,
        every subscription must be unfinished at first
        """

        workflowName = 'Express_Run481516_StreamZFast'
        secondLevelTasks = ['ExpressMergewrite_StreamZFast_DQM', 'ExpressMergewrite_ExpressPhysics_FEVT',
                            'ExpressAlcaSkimwrite_StreamZFast_ALCARECO', 'ExpressCleanupUnmergedwrite_StreamZFast_DQM',
                            'ExpressCleanupUnmergedwrite_ExpressPhysics_FEVT',
                            'ExpressCleanupUnmergedwrite_StreamZFast_ALCARECO']
        alcaHarvestTask = 'ExpressAlcaSkimwrite_StreamZFast_ALCARECOAlcaHarvestALCARECOStreamPromptCalibProd'
        dqmHarvestTask = 'ExpressMergewrite_StreamZFast_DQMEndOfRunDQMHarvestMerged'

        self.stateMap = {'Merge': [],
                         'Harvesting': [],
                         'Processing Done': []}
        self.orderedStates = ['Merge', 'Harvesting', 'Processing Done']

        # Populate WMStats
        self.requestDBWriter.insertGenericRequest({'RequestName': workflowName})
        self.requestDBWriter.updateRequestStatus(workflowName, 'Closed')

        # Create a wmspec in disk
        workload = newWorkload(workflowName)
        expressTask = workload.newTask('Express')
        for task in secondLevelTasks:
            secondLevelTask = expressTask.addTask(task)
            if task == 'ExpressAlcaSkimwrite_StreamZFast_ALCARECO':
                secondLevelTask.addTask(alcaHarvestTask)
            elif task == 'ExpressMergewrite_StreamZFast_DQM':
                secondLevelTask.addTask(dqmHarvestTask)

        specPath = os.path.join(self.testDir, 'Express.pkl')
        workload.save(specPath)

        # Populate WMBS
        sharedFileset = Fileset(name='TestFileset')
        sharedFileset.create()
        sharedFileset.markOpen(False)

        options = {'spec': specPath, 'owner': 'ItsAMeMario',
                   'name': workflowName, 'wfType': 'tier0'}
        topLevelWorkflow = Workflow(task='/%s/Express' % workflowName,
                                    **options)
        topLevelWorkflow.create()
        topLevelSub = Subscription(sharedFileset, topLevelWorkflow)
        topLevelSub.create()
        self.stateMap['Merge'].append(topLevelSub)
        for task in [x for x in secondLevelTasks if not x.count('CleanupUnmerged')]:
            secondLevelWorkflow = Workflow(task='/%s/Express/%s' % (workflowName, task), **options)
            secondLevelWorkflow.create()
            mergeSub = Subscription(sharedFileset, secondLevelWorkflow)
            mergeSub.create()
            self.stateMap['Harvesting'].append(mergeSub)

        for (parent, child) in [('ExpressAlcaSkimwrite_StreamZFast_ALCARECO', alcaHarvestTask),
                                ('ExpressMergewrite_StreamZFast_DQM', dqmHarvestTask)]:
            harvestingWorkflow = Workflow(task='/%s/Express/%s/%s' % (workflowName, parent, child),
                                          **options)
            harvestingWorkflow.create()
            harvestingSub = Subscription(sharedFileset, harvestingWorkflow)
            harvestingSub.create()
            self.stateMap['Processing Done'].append(harvestingSub)

        return

    def setupPromptRecoWorkflow(self):
        """
        _setupPromptRecoWorkflow_

        Populate WMBS with a real PromptReco workflow,
        every subscription must be unfinished at first
        """

        # Populate disk and WMBS
        testArguments = PromptRecoWorkloadFactory.getTestArguments()

        workflowName = 'PromptReco_Run195360_Cosmics'
        factory = PromptRecoWorkloadFactory()
        testArguments["EnableHarvesting"] = True
        testArguments["CouchURL"] = os.environ["COUCHURL"]
        workload = factory.factoryWorkloadConstruction(workflowName, testArguments)

        wmbsHelper = WMBSHelper(workload, 'Reco', 'SomeBlock', cachepath=self.testDir)
        wmbsHelper.createTopLevelFileset()
        wmbsHelper._createSubscriptionsInWMBS(wmbsHelper.topLevelTask, wmbsHelper.topLevelFileset)

        self.stateMap = {'AlcaSkim': [],
                         'Merge': [],
                         'Harvesting': [],
                         'Processing Done': []}
        self.orderedStates = ['AlcaSkim', 'Merge', 'Harvesting', 'Processing Done']

        # Populate WMStats
        self.requestDBWriter.insertGenericRequest({'RequestName': workflowName})
        self.requestDBWriter.updateRequestStatus(workflowName, 'Closed')

        topLevelTask = '/%s/Reco' % workflowName
        alcaSkimTask = '%s/AlcaSkim' % topLevelTask
        mergeTasks = ['%s/AlcaSkim/AlcaSkimMergeALCARECOStreamHcalCalHOCosmics',
                      '%s/AlcaSkim/AlcaSkimMergeALCARECOStreamTkAlCosmics0T',
                      '%s/AlcaSkim/AlcaSkimMergeALCARECOStreamMuAlGlobalCosmics',
                      '%s/RecoMergewrite_AOD',
                      '%s/RecoMergewrite_DQM',
                      '%s/RecoMergewrite_RECO']
        harvestingTask = '%s/RecoMergewrite_DQM/RecoMergewrite_DQMEndOfRunDQMHarvestMerged' % topLevelTask

        self.stateMap['AlcaSkim'].append(wmbsHelper.topLevelSubscription)

        alcaSkimWorkflow = Workflow(name=workflowName, task=alcaSkimTask)
        alcaSkimWorkflow.load()
        alcarecoFileset = Fileset(name='/PromptReco_Run195360_Cosmics/Reco/unmerged-write_ALCARECOALCARECO')
        alcarecoFileset.load()
        alcaSkimSub = Subscription(alcarecoFileset, alcaSkimWorkflow)
        alcaSkimSub.load()
        self.stateMap['Merge'].append(alcaSkimSub)

        for task in mergeTasks:
            mergeTask = task % topLevelTask
            mergeWorkflow = Workflow(name=workflowName, task=mergeTask)
            mergeWorkflow.load()
            if 'AlcaSkim' in mergeTask:
                stream = mergeTask.split('/')[-1][13:]
                unmergedFileset = Fileset(name='%s/unmerged-%sALCARECO' % (alcaSkimTask, stream))
                unmergedFileset.load()
            else:
                dataTier = mergeTask.split('/')[-1].split('_')[-1]
                unmergedFileset = Fileset(name='%s/unmerged-write_%s%s' % (topLevelTask, dataTier, dataTier))
                unmergedFileset.load()
            mergeSub = Subscription(unmergedFileset, mergeWorkflow)
            mergeSub.load()
            self.stateMap['Harvesting'].append(mergeSub)

        harvestingWorkflow = Workflow(name=workflowName, task=harvestingTask)
        harvestingWorkflow.load()
        harvestingFileset = Fileset(name='/PromptReco_Run195360_Cosmics/Reco/RecoMergewrite_DQM/merged-MergedDQM')
        harvestingFileset.load()
        harvestingSub = Subscription(harvestingFileset, harvestingWorkflow)
        harvestingSub.load()
        self.stateMap['Processing Done'].append(harvestingSub)

        return

    def verifyStateTransitions(self, transitionMethod='markFinished', transitionTrigger=True):
        """
        _verifyStateTransitions_

        Utility method which goes through the list of states in self.orderedStates and
        finishes the tasks that demand a state transition in each step. This according
        to the defined transition method and trigger.
        It verifies that the request document in WMStats is moving according to the transitions
        """

        for idx in range(0, len(self.orderedStates) * 2):
            nextState = self.orderedStates[idx // 2]
            if (idx // 2) == 0:
                currentState = 'Closed'
            else:
                currentState = self.orderedStates[idx // 2 - 1]
            if idx % 2 == 0:
                for transitionObject in self.stateMap[nextState][:-1]:
                    method = getattr(transitionObject, transitionMethod)
                    method(transitionTrigger)
                self.plugin([], self.requestDBWriter, self.requestDBWriter)
                currentStateWorkflows = self.requestDBWriter.getRequestByStatus([currentState])
                nextStateWorkflows = self.requestDBWriter.getRequestByStatus([nextState])
                self.assertEqual(len(currentStateWorkflows), 1, 'Workflow moved incorrectly from %s' % currentState)
                self.assertEqual(len(nextStateWorkflows), 0, 'Workflow moved incorrectly to %s' % nextState)
            else:
                transitionObject = self.stateMap[nextState][-1]
                method = getattr(transitionObject, transitionMethod)
                method(transitionTrigger)
                self.plugin([], self.requestDBWriter, self.requestDBWriter)
                currentStateWorkflows = self.requestDBWriter.getRequestByStatus([currentState])
                nextStateWorkflows = self.requestDBWriter.getRequestByStatus([nextState])
                self.assertEqual(len(currentStateWorkflows), 0,
                                 'Workflow did not move correctly from %s' % currentState)
                self.assertEqual(len(nextStateWorkflows), 1, 'Workflow did not move correctly to %s' % nextState)
        return

    def testA_RepackStates(self):
        """
        _testA_RepackStates_

        Setup an environment with a Repack workflow
        and traverse through the different states.
        Check that the transitions are sane.
        """
        # Set the environment
        self.setupRepackWorkflow()
        self.plugin = Tier0Plugin()

        # Verify the transitions
        self.verifyStateTransitions('markOpen', False)

        return

    def testB_ExpressStates(self):
        """
        _testB_ExpressStates_

        Setup an environment with a Express workflow
        and traverse through the different states.
        Check that the transitions are sane.
        """
        # Set the environment
        self.setupExpressWorkflow()
        self.plugin = Tier0Plugin()

        # Verify the transitions
        self.verifyStateTransitions()

        return

    def testC_PromptRecoStates(self):
        """
        _testC_PromptRecoStates_

        Setup an environment with a PromptReco workflow
        and traverse through the different states.
        Check that the transitions are sane.
        """
        # Set the environment
        self.setupPromptRecoWorkflow()
        self.plugin = Tier0Plugin()

        # Verify the transitions
        self.verifyStateTransitions()

        return


if __name__ == "__main__":
    unittest.main()
